@@ -1,9 +1,12 @@
 import asyncio
+import contextlib
 import errno
+import select
 import sys
 import traceback
 import warnings
 import weakref
+import platform
 
 import psycopg2
 from psycopg2.extensions import (
@@ -20,6 +23,10 @@ __all__ = ('connect',)
 TIMEOUT = 60.0
 PY_341 = sys.version_info >= (3, 4, 1)
 
+# Windows specific error code, not in errno for some reason, and doesnt map
+# to OSError.errno EBADF
+WSAENOTSOCK = 10038
+
 
 @asyncio.coroutine
 def _enable_hstore(conn):
@@ -80,6 +87,13 @@ def _connect(dsn=None, *, timeout=TIMEOUT, loop=None, enable_json=True,
     return conn
 
 
+def _is_bad_descriptor_error(os_error):
+    if platform.system() == 'Windows':  # pragma: no cover
+        return os_error.winerror == WSAENOTSOCK
+    else:
+        return os_error.errno == errno.EBADF
+
+
 class Connection:
     """Low-level asynchronous interface for wrapped psycopg2 connection.
 
@@ -99,6 +113,8 @@ class Connection:
         self._timeout = timeout
         self._waiter = waiter
         self._writing = False
+        self._cancelling = False
+        self._cancellation_waiter = None
         self._echo = echo
         self._notifies = asyncio.Queue(loop=loop)
         self._weakref = weakref.ref(self)
@@ -120,10 +136,22 @@ class Connection:
                 notify = self._conn.notifies.pop(0)
                 self._notifies.put_nowait(notify)
         except (psycopg2.Warning, psycopg2.Error) as exc:
+            if self._fileno is not None:
+                try:
+                    select.select([self._fileno], [], [], 0)
+                except OSError as os_exc:
+                    if _is_bad_descriptor_error(os_exc):
+                        with contextlib.suppress(OSError):
+                            self._loop.remove_reader(self._fileno)
+                            # forget a bad file descriptor, don't try to
+                            # touch it
+                            self._fileno = None
+
             try:
                 if self._writing:
                     self._writing = False
-                    self._loop.remove_writer(self._fileno)
+                    if self._fileno is not None:
+                        self._loop.remove_writer(self._fileno)
             except OSError as exc2:
                 if exc2.errno != errno.EBADF:
                     # EBADF is ok for closed file descriptor
@@ -133,6 +161,11 @@ class Connection:
             if waiter is not None and not waiter.done():
                 waiter.set_exception(exc)
         else:
+            if self._fileno is None:
+                # connection closed
+                if waiter is not None and not waiter.done():
+                    waiter.set_exception(
+                        psycopg2.OperationalError("Connection closed"))
             if state == POLL_OK:
                 if self._writing:
                     self._loop.remove_writer(self._fileno)
@@ -168,8 +201,14 @@ class Connection:
 
     def _create_waiter(self, func_name):
         if self._waiter is not None:
-            raise RuntimeError('%s() called while another coroutine is '
-                               'already waiting for incoming data' % func_name)
+            if self._cancelling:
+                if not self._waiter.done():
+                    raise RuntimeError('%s() called while connection is '
+                                       'being cancelled' % func_name)
+            else:
+                raise RuntimeError('%s() called while another coroutine is '
+                                   'already waiting for incoming '
+                                   'data' % func_name)
         self._waiter = create_future(self._loop)
         return self._waiter
 
@@ -180,22 +219,35 @@ class Connection:
 
         @asyncio.coroutine
         def cancel():
-            if not self._isexecuting():
-                return
             self._waiter = create_future(self._loop)
+            self._cancelling = True
+            self._cancellation_waiter = self._waiter
             self._conn.cancel()
+            if not self._conn.isexecuting():
+                return
             try:
-                yield from self._waiter
+                yield from asyncio.wait_for(self._waiter, timeout,
+                                            loop=self._loop)
             except psycopg2.extensions.QueryCanceledError:
                 pass
+            except asyncio.TimeoutError:
+                self._close()
 
         try:
             yield from asyncio.wait_for(self._waiter, timeout, loop=self._loop)
         except (asyncio.CancelledError, asyncio.TimeoutError) as exc:
             yield from asyncio.shield(cancel(), loop=self._loop)
             raise exc
+        except psycopg2.extensions.QueryCanceledError:
+            raise asyncio.CancelledError
         finally:
-            self._waiter = None
+            if self._cancelling:
+                self._cancelling = False
+                if self._waiter is self._cancellation_waiter:
+                    self._waiter = None
+                self._cancellation_waiter = None
+            else:
+                self._waiter = None
 
     def _isexecuting(self):
         return self._conn.isexecuting()
@@ -240,18 +292,22 @@ class Connection:
                                      scrollable=scrollable, withhold=withhold)
         return impl
 
-    def close(self):
+    def _close(self):
         """Remove the connection from the event_loop and close it."""
         # N.B. If connection contains uncommitted transaction the
         # transaction will be discarded
-        self._loop.remove_reader(self._fileno)
-        if self._writing:
-            self._writing = False
-            self._loop.remove_writer(self._fileno)
+        if self._fileno is not None:
+            self._loop.remove_reader(self._fileno)
+            if self._writing:
+                self._writing = False
+                self._loop.remove_writer(self._fileno)
         self._conn.close()
         if self._waiter is not None and not self._waiter.done():
             self._waiter.set_exception(
                 psycopg2.OperationalError("Connection closed"))
+
+    def close(self):
+        self._close()
         ret = create_future(self._loop)
         ret.set_result(None)
         return ret
@@ -313,19 +369,13 @@ class Connection:
             "tpc_recover cannot be used in asynchronous mode")
 
     @asyncio.coroutine
-    def cancel(self, timeout=None):
+    def cancel(self):
         """Cancel the current database operation."""
-        if timeout is not None:
-            warnings.warn('timeout parameter is deprecated and never used',
-                          DeprecationWarning)
-        if self._waiter is not None:
-            self._waiter.cancel()
-        if not self._isexecuting():
+        if self._waiter is None:
             return
 
         @asyncio.coroutine
         def cancel():
-            self._waiter = create_future(self._loop)
             self._conn.cancel()
             try:
                 yield from self._waiter
